local AddonName, GGL = ...
--[[-----------------------------------------------------------------------------
Label Widget
Displays text and optionally an icon.
-------------------------------------------------------------------------------]]
local Type, Version = "GoldGoblinLootGUIWidget-OpeningItem", 1
local AceGUI = LibStub and LibStub("AceGUI-3.0", true)
if not AceGUI or (AceGUI:GetWidgetVersion(Type) or 0) >= Version then return end

-- Lua APIs
local max, select, pairs = math.max, select, pairs
local tonumber = tonumber
local abs, floor = math.abs, math.floor
local format = string.format
local tbl_remove = table.remove

-- WoW APIs
local CreateFrame, UIParent = CreateFrame, UIParent
local GetItemIcon, GetItemInfo = GetItemIcon, GetItemInfo

--[[-----------------------------------------------------------------------------
Support functions
-------------------------------------------------------------------------------]]
local FRAME_HEIGHT, FRAME_GAP = 40, 2

local MAIN_SOURCE_FLAG_ICONS = GGL.MAIN_SOURCE_FLAG_ICONS
local SUB_SOURCE_FLAG_ICONS = GGL.SUB_SOURCE_FLAG_ICONS
local MAIN_SOURCE_FLAG = GGL.MAIN_SOURCE_FLAG

local ITEM_NUM_FORMAT = "%s x %d"

local GetMoneyText = GGL.GetMoneyTextShort

local function Frame_OnEnter(self)
    if not self.obj.item then return end

    local tooltip = _G.GameTooltip
    tooltip:ClearLines()
    GGL.GUI:SetToolTipPoint(tooltip, self.obj.ttParent, self.obj.ttPoint, 2, 0)
    if type(self.obj.item) == "string" then
        tooltip:SetHyperlink(self.obj.item)
    else
        tooltip:SetItemByID(self.obj.item)
    end
    tooltip:Show()
end

local function Frame_OnLeave(self)
    _G.GameTooltip:Hide()
end

local function Frame_OnClick(self, button)
    if not self.obj.item  then return end
    GGL:ItemMouseOnClick(self.obj.item, self, button)
    self.obj:SetSelected(not self.obj.isSelected)
end

--[[-----------------------------------------------------------------------------
Methods
-------------------------------------------------------------------------------]]
local methods = {
    ["OnAcquire"] = function(self)
        self:ResetToolTipPoint()
        self:SetItemData()
        self:SetSelected(false)
        self:DisableClick()
        self.frame:SetHeight(FRAME_HEIGHT)
        self.frame:Show()
    end,

    ["OnRelease"] = function(self)
        self.frame:Hide()
	end,

    ["OnWidthSet"] = function(self, width)

    end,

    ["SetToolTipPoint"] = function(self, ttPoint, parent)
        self.ttPoint = ttPoint or "BOTTOM_X"
        self.ttParent = parent or self.frame
    end,

    ["ResetToolTipPoint"] = function(self)
        self.ttPoint = "BOTTOM_X"
        self.ttParent = self.frame
    end,

    --- Updates the item with the item data
    ["UpdateItem"] = function(self)
        if not self.item then
            self.ammount = nil
            self.goldValue = nil
            self.setItem = nil
            self.itemLink = nil
            return
        end

        local item = self.item

        self.itemAmmountText:SetText(self.ammount or 0)
        self.itemMoneyText:SetFormattedText(GetMoneyText(self.goldValue))

        -- some things only change with new itemID
        if not self.itemLink or self.setItem ~= item then
            local itemName, itemLink = GetItemInfo(item)
            if not itemLink then
                GGL:IsItemCached(item, self.UpdateItem, self)
            end
            self.itemLink = itemLink
            self.itemNameText:SetText(itemLink or "")
            self.itemIcon:SetTexture(GetItemIcon(item))

            self.setItem = item
        end
    end,

    ["GetSortValue"] = function(self)
        return self.goldValue or 0
    end,

    --- Sets the item data from TopItems
    ["SetItemData"] = function(self, item, ammount, goldValue)
        self.item = item
        self.ammount = ammount
        self.goldValue = goldValue
        self:UpdateItem()
    end,

    ["DisableClick"] = function(self, state)
        self.disableClick = state
    end,

    ["SetSelected"] = function(self, state, blockEvent)
        if self.disableClick then return end
        local stateChange = state ~= self.isSelected
        self.isSelected = state
        if state then
            self.frameBG:Show()
        else
            self.frameBG:Hide()
        end
        if not blockEvent and stateChange then
            self:Fire("SelectStateChange", state)
        end
    end,
}

--[[-----------------------------------------------------------------------------
Constructor
-------------------------------------------------------------------------------]]
local function Constructor()
    local widget = {}
    widget.type  = Type

    local frame = CreateFrame("Button", nil, UIParent)
    frame:SetHeight(FRAME_HEIGHT)
    frame:SetScript("OnEnter", Frame_OnEnter)
    frame:SetScript("OnLeave", Frame_OnLeave)
    frame:SetScript("OnClick", Frame_OnClick)
    frame.obj = widget
    widget.frame = frame

    -- background image for list position
    local frameBG = frame:CreateTexture(nil, "BACKGROUND", nil, -1)
    frameBG:SetAllPoints(frame)
    frameBG:SetColorTexture(0, 0.82, 1, 0.25)
    frameBG:Hide()
    widget.frameBG = frameBG

    -- itemIcon
    local itemIcon = frame:CreateTexture(nil, "BACKGROUND")
    itemIcon:SetPoint("TOPLEFT", frame, "TOPLEFT", FRAME_GAP, -FRAME_GAP)
    itemIcon:SetSize(FRAME_HEIGHT-(FRAME_GAP*2), FRAME_HEIGHT-(FRAME_GAP*2))
    widget.itemIcon = itemIcon

    -- itemNameText
    local itemNameText = frame:CreateFontString(nil, "BACKGROUND", "GameFontHighlight")
    itemNameText:SetPoint("TOP", frame, "TOP", 0, -FRAME_GAP)
    itemNameText:SetPoint("LEFT", itemIcon, "RIGHT", FRAME_GAP, 0)
    itemNameText:SetPoint("RIGHT", frame, "RIGHT", -FRAME_GAP, 0)
    itemNameText:SetHeight(15)
    itemNameText:SetJustifyH("LEFT")
    itemNameText:SetJustifyV("MIDDLE")
    widget.itemNameText = itemNameText

    -- itemMoneyText
    local itemMoneyText = frame:CreateFontString(nil, "BACKGROUND", "GameFontHighlight")
    itemMoneyText:SetPoint("TOPLEFT", itemNameText, "BOTTOMLEFT", 0, -FRAME_GAP)
    itemMoneyText:SetPoint("TOPRIGHT", itemNameText, "BOTTOMRIGHT", 0, -FRAME_GAP)
    itemMoneyText:SetPoint("BOTTOM", itemIcon, "BOTTOM", 0, 0)
    itemMoneyText:SetJustifyH("RIGHT")
    itemMoneyText:SetJustifyV("TOP")
    widget.itemMoneyText = itemMoneyText

    -- itemAmmountText
    local itemAmmountText = frame:CreateFontString(nil, "BACKGROUND", "GameFontHighlight")
    itemAmmountText:SetPoint("TOPLEFT", itemNameText, "BOTTOMLEFT", 0, -FRAME_GAP)
    itemAmmountText:SetPoint("TOPRIGHT", itemNameText, "BOTTOMRIGHT", 0, -FRAME_GAP)
    itemAmmountText:SetPoint("BOTTOM", itemIcon, "BOTTOM", 0, 0)
    itemAmmountText:SetJustifyH("LEFT")
    itemAmmountText:SetJustifyV("MIDDLE")
    widget.itemAmmountText = itemAmmountText

    -- methodes
    for method, func in pairs(methods) do
        widget[method] = func
    end

    return AceGUI:RegisterAsWidget(widget)
end

AceGUI:RegisterWidgetType(Type, Constructor, Version)